#include "game.h"
#include "buzzer.h"
#include "mpu6050/mpu6050.h"

#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>

volatile int start_game = 0;
int32_t random_number;
enum t_bool first_time = TRUE;

/*
 * Start the game when button BTN is pressed
 * -> random number generator is initialised based
 *	on value of random_number
 */
ISR(PCINT3_vect)
{
	if ((PIND & (1 << PD6)) == 0) {
		random_number = TCNT1;
		start_game = 1;
		PORTD |= (1 << PD7);
		_delay_ms(100);
		PORTD &= ~(1 << PD7);
		_delay_ms(100);
	}
}

/*
 * OCR1A compare interrupt 
 */
ISR(TIMER1_COMPA_vect)
{
	/*	PORTD |= (1 << PD7);
		_delay_ms(100);
		PORTD &= ~(1 << PD7);
		_delay_ms(100);
	*/
}

/*
 * Initialize timer1 in order to generate random_number
 * based on value of reg TCNT1
 */
void timer1_init()
{
	// max value
    OCR1A = 15625;

    // activate OCR1A compare interrupt mode
    TIMSK1 = (1 << OCIE1A);
    
    // mode CTC - OCR1A, prescaler: 1024
    TCCR1B = (5 << CS10) | (1 << WGM12);
}

void timer1_clean_up()
{
    TIMSK1 &= ~(1 << OCIE1A);
    TCCR1B &= ~(5 << CS10);
    TCCR1B &= ~(1 << WGM12);
}

ISR(TIMER0_COMPA_vect)
{
	game_set_direction();
}

void timer0_init()
{
	// max value
    OCR0A = 32;
    // activate OCR0A compare interrupt mode
    TIMSK0 = (1 << OCIE0A);
    
   	// CTC, top OCRA
	TCCR0B |= (0 << WGM02);
	TCCR0A |= (1 << WGM01) | (0 << WGM00);

	// prescaler 1024
	TCCR0B |= (5 << CS00);
}

void BTN_init()
{
	// set BTN && activate pull-up for BTN
	DDRD &= ~(1 << PD6);
	PORTD |= (1 << PD6);
}

void interrupts_init()
{
	// PD6 pin interrupt
    PCICR  = (1 << PCIE3);
    PCMSK3 = (1 << PCINT30);
}


int main (void)
{

    sei();
	w_lcd_init();
	mpu6050_init();
	buzzer_init();
	timer0_init();
	timer1_init();
	BTN_init();
    interrupts_init();

here:
    srand(0);
    // wait for player to press BTN -> generate
    // random number used to initialize srand
	while (start_game == 0) {
		// effect
		if(first_time) {
			initial_effect();
		}
		w_lcd_print_press_btn_effect(first_time);
		first_time = FALSE;
	}
	srand(random_number);
	w_lcd_clear_screen();
	timer1_clean_up();

	// start game	
	game_start();

	// finish game
	w_lcd_clear_screen();
start_game = 0;
first_time = TRUE;
goto here;

	buzzer_clean();

	return 0;
}
