#include "game.h"

void game_set_direction() {
	double axg, ayg, azg, gxds, gyds, gzds;

	axg = ayg = azg = 0;
	mpu6050_getConvData(&axg, &ayg, &azg, &gxds, &gyds, &gzds);

	if (ayg > LIMIT) {
		ring[ring_index] = DOWN;
	} else if (ayg < -LIMIT) {
		ring[ring_index] = UP;
	}

	if (axg > LIMIT) {
		ring[ring_index] = LEFT;
	} else if (axg < -LIMIT) {
		ring[ring_index] = RIGHT;
	}

	ring_index ++;
	ring_index %= RING_SIZE;
}

void game_init()
{
	int16_t i, j = 0;
	struct t_cell c1, c2;
	enum t_bool ret;

	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			gm[i][j] = (int16_t) 0;

	ret = gen_rand_cell(&c1);
	if (ret == FALSE)
		return;
	ret = gen_rand_cell(&c2);
	if (ret == FALSE)
		return;
	
	draw_cell(c1, get_value(c1));
	draw_cell(c2, get_value(c2));

	for (i = 0; i < RING_SIZE; i++)
		ring[i] = NO_DIR;
}

void move_left_handler(int32_t matrix[N][N])
{
	int16_t i, j, k;
	enum t_bool swap = FALSE;

	for (j = 0; j < N; j++) {
		swap = FALSE;

		for (i = 0; i < N; i++) {
			if (matrix[i][j] == 0) {
				swap = TRUE;

				for (k = i + 1; k < N; k++) {
					if (matrix[k][j] != 0) {
						matrix[i][j] = matrix[k][j];
						matrix[k][j] = 0;
						break;
					}
				}
			}
		}

		if (swap == TRUE) {
			for (i = 0; i < N - 1; i++) { 
				if (matrix[i][j] != 0) {
					if (matrix[i][j] == matrix[i+1][j]) {
						matrix[i][j] = matrix[i][j] << 1;
						matrix[i+1][j] = 0;
					}
				}
			}
		}

		if (swap == FALSE) {
			if ((matrix[0][j] == matrix[1][j]) && (matrix[2][j] == matrix[3][j])) {
				//2 2 4 4 sau 2 2 2 2 => 4 16 0 0 sau 4 4 0 0
				matrix[0][j] = matrix[0][j] << 1;
				matrix[1][j] = matrix[2][j] << 1;
				matrix[2][j] = matrix[3][j] = 0;

			} else if (matrix[0][j] == matrix[1][j] && (matrix[2][j] != matrix[3][j])) {
				//2 2 4 8 => 4 4 8 0
				
				matrix[0][j] = matrix[0][j] << 1;
				matrix[1][j] = matrix[2][j];
				matrix[2][j] = matrix[3][j];
				matrix[3][j] = 0;

			} else if ((matrix[0][j] != matrix[1][j]) && matrix[2][j] == matrix[3][j]) {
				// 2 4 8 8 => 2 4 16 0

				matrix[2][j] = matrix[2][j] << 1;
				matrix[3][j] = 0;

			} else if (matrix[1][j] == matrix[2][j] && (matrix[0][j] != matrix[1][j])) {
				// 2 4 4 2 => 2 8 2 0 

				matrix[1][j] = matrix[1][j] << 1;
				matrix[2][j] = matrix[3][j];
				matrix[3][j] = 0;

			}
		}
	}
}

void move_right_handler(int32_t matrix[N][N])
{
	// rotate the game matrix with 180 degrees and then apply the same alghoritm
	//for move_up
	int32_t rotate_game_matrix_180deg[N][N];
	int16_t i, j;

	// i rotate the matrix with 180 degrees
	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			rotate_game_matrix_180deg[i][j] = matrix[N - i - 1][j];
	
	// move up the rotated matrix
	move_left_handler(rotate_game_matrix_180deg);
	
	// rotate back the matrix (+/- 180 degrees)
	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			matrix[i][j] = rotate_game_matrix_180deg[N - i - 1][j];
}

void move_up_handler(int32_t matrix[N][N])
{
	// rotate the matrix with +90 degrees
	int32_t rotate_game_matrix_plus90deg[N][N];
	int16_t i, j;

	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			rotate_game_matrix_plus90deg[i][j] = matrix[N - j - 1][i];
	// move up the rotated matrix
	move_left_handler(rotate_game_matrix_plus90deg);
	
	// rotate back the matrix (-90 degrees)
	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			matrix[i][j] = rotate_game_matrix_plus90deg[j][N - i - 1];
}

void move_down_handler(int32_t matrix[N][N])
{
	// rotate the matrix with -90 degrees
	int32_t rotate_game_matrix_minus90deg[N][N];
	int16_t i, j;

	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			rotate_game_matrix_minus90deg[i][j] =  matrix[j][N - i - 1];
	
	// move up the rotated matrix
	move_left_handler(rotate_game_matrix_minus90deg);
	
	// rotate back the matrix (+90 degrees)
	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			matrix[i][j] = rotate_game_matrix_minus90deg[N - j - 1][i];
}

void move_up()
{
	move_up_handler(gm);
}

void move_down()
{
	move_down_handler(gm);
}

void move_left()
{
	move_left_handler(gm);
}

void move_right()
{
	move_right_handler(gm);
}

// obtain number of free position in gm
int16_t count_free_cells(int32_t matrix[N][N]) {
	int16_t i, j, count = 0;

	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			if (matrix[i][j] == 0)
				count += 1;

	return count;
}

// return a copy of gm
void copy_gm(int32_t cpgm[N][N]) {
	int16_t i, j;

	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			cpgm[i][j] = gm[i][j];
}

enum t_state move(void (*move_handler)())
{
	struct t_cell cell;
	enum t_bool ret;
	enum t_state state = CONTINUE;

	move_handler();
	buzzer_play_move();
	
	if (check_win() == TRUE)
		state = WIN;

	print_game();

	// draw new random cell
	ret = gen_rand_cell(&cell);
	if (ret == FALSE)
		state = LOST;

	_delay_ms(100);
	draw_cell(cell, get_value(cell));

	return state;
}

enum t_state best_local_move() {
	int32_t fgm[N][N];
	int16_t fpu, fpd, fpl, fpr, fpmax;
	enum t_state fs = CONTINUE;

	// trying to mode up
	copy_gm(fgm);
	move_up_handler(fgm);
	fpu = count_free_cells (fgm);

	// trying to move down
	copy_gm(fgm);
	move_down_handler(fgm);
	fpd = count_free_cells (fgm);

	// trying to move left
	copy_gm(fgm);
	move_left_handler(fgm);
	fpl = count_free_cells (fgm);
	
	// trying to move right
	copy_gm(fgm);
	move_right_handler(fgm);
	fpr = count_free_cells (fgm);
	
	// getting the move which free the maximum size of the game matrix
	fpmax = fpd;
	if (fpu > fpmax)
		fpmax = fpu;
	if (fpl > fpmax)
		fpmax = fpl;
	if (fpr > fpmax)
		fpmax = fpr;

	// take the best move (which free the greatest space of game matrix) 
	if (fpmax == fpu)
		fs = move(move_up);
	else if (fpmax == fpd)
		fs = move(move_down);
	else if (fpmax == fpl)
		fs = move(move_left);
	else if (fpmax == fpr)
		fs = move(move_right);

	return fs;
}

enum t_dir get_direction()
{
	int16_t i;
	enum t_dir dir0 = ring[0];

	for (i = 1; i < RING_SIZE; i++) {
		if (ring[i] != dir0)
			return NO_DIR;
	}

	return dir0;
}

enum t_bool game_start()
{
	int16_t i;
	struct t_cell cell;
	enum t_state ret = CONTINUE;
	enum t_dir dir = NO_DIR;

	game_init();

	if(!AUTOPLAY) {

		while (1) {
			dir = get_direction();
			drawPixel(0, 0, Color565(255,255, 255));

			//test_ring_mpu();

			if (dir != NO_DIR) {
				if (dir == UP) {
					ret = move(move_up);
				} else if (dir == DOWN) {
					ret = move(move_down);
				} else if (dir == LEFT) {
					ret = move(move_left);
				} else if (dir == RIGHT) {
					ret = move(move_right);
				}
				if (ret == LOST || ret == WIN)
					break;
				_delay_ms(700);
			}
		}

	} else {
		while (1) {
			ret = best_local_move();
			if (ret == LOST || ret == WIN)
				break;
			_delay_ms(500);
		}
	}


	if (ret == WIN) {
		buzzer_play_win();
	} else if (ret == LOST) {
		buzzer_play_lost();
	}

}

void print_game()
{
	int16_t i, j = 0;
	struct t_cell cell;

	// delete cells
	for (i = 0; i < N; i++) {
		for (j = 0; j < N; j++) {
			cell.x = i;
			cell.y = j;

			if (gm[i][j] == (int16_t) 0) {
				clean_cell(cell);
			}
		}
	}

	// print new cells
	for (i = 0; i < N; i++) {
		for (j = 0; j < N; j++) {
			cell.x = i;
			cell.y = j;

			if (gm[i][j] != (int16_t) 0) {
				draw_cell(cell, get_value(cell));
			}
		}
	}

}

int32_t get_value(struct t_cell cell)
{
	if (cell.x < 0 || cell.x >= N)
		return -1;
	if (cell.y < 0 || cell.y >= N)
		return -1;

	return gm[cell.x][cell.y];
}

enum t_bool check_win()
{
	int16_t i, j = 0;

	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			if (gm[i][j] == (int32_t) WIN_CONDITION)
				return TRUE;

	return FALSE;
}

enum t_bool exists_free_cell()
{
	int16_t i, j = 0;

	for (i = 0; i < N; i++)
		for (j = 0; j < N; j++)
			if (gm[i][j] == (int32_t) 0)
				return TRUE;

	return FALSE;
}

enum t_bool check_free_cell(struct t_cell *cell) {

	if (gm[cell->x][cell->y] == (int32_t) 0)
		return TRUE;

	return FALSE;
}

enum t_bool gen_rand_cell(struct t_cell *cell)
{
	int32_t value;
	
	cell->x = rand() % N;
	cell->y = rand() % N;

	if (!exists_free_cell()) {
		while(1)
			PORTD |= (1 << PD7);
		return FALSE;
	}

	while (check_free_cell(cell) == FALSE) {
		cell->x = rand() % N;
		cell->y = rand() % N;
	}

	value = (int32_t) (rand() % 4);
	if (value == 0 || value == 1 || value == 2)
		gm[cell->x][cell->y] = (int32_t) 2;
	else
		gm[cell->x][cell->y] = (int32_t) 4;

	return TRUE;
}

static void test_ring_mpu()
{
	struct t_point p1, p2;

	p1.x = border[0] + 1;
 	p1.y = border[0] + 1;
 	p2.x = border[1] - 1;
 	p2.y = border[1] - 1;
 	w_lcd_draw_square(p1, p2, ST7735_RED, Color565(255,255, 255), ring[0]);

 	p1.x = border[1] + 1;
 	p1.y = border[0] + 1;
 	p2.x = border[2] - 1;
 	p2.y = border[1] - 1;
 	w_lcd_draw_square(p1, p2, ST7735_RED, Color565(255,255, 255), ring[1]);

	p1.x = border[2] + 1;
 	p1.y = border[0] + 1;
 	p2.x = border[3] - 1;
 	p2.y = border[1] - 1;
 	w_lcd_draw_square(p1, p2, ST7735_RED, Color565(255,255, 255), ring[2]);
}
