#include "w_lcd.h"

//void drawPixel(int16_t x, int16_t y, uint16_t color) {

void w_lcd_init()
{
	lcd_init();
    fillScreen(Color565(255,255,255));
}

void w_lcd_clear_screen()
{	
    fillScreen(Color565(255,255,255));
}

void w_lcd_draw_square (struct t_point p1, struct t_point p2,
						uint16_t color, uint16_t border_color,
						int32_t value)
{
	int16_t min_x, min_y, max_x, max_y;
	int16_t i, j;
	struct t_point value_pos;

	if (INVERTED_MODE) {
		p1 = get_inverted_point(p1);
		p2 = get_inverted_point(p2);
	}
	
	if (p1.x < 0 || p1.x >= w_width || p1.y < 0 || p1.y >= w_height)
		return;
	if (p2.x < 0 || p2.x >= w_width || p2.y < 0 || p2.y >= w_height)
		return;

	min_x = min(p1.x, p2.x);
	max_x = max(p1.x, p2.x);
	min_y = min(p1.y, p2.y);
	max_y = max(p1.y, p2.y);

	for (i = min_x; i <= max_x; i++)
		for (j = min_y; j <= max_y; j++)
			if(i == min_x || i == max_x || j == min_y || j == max_y)
				drawPixel(i, j, border_color);
			else
				drawPixel(i, j, color);

	if (value != -1) {
		value_pos.x = (min_x + max_x) / 2;
		value_pos.y = (min_y + max_y) / 2;
		w_lcd_print_value (value_pos, value);
	}
}

void w_lcd_print_value (struct t_point pos, int32_t value)
{
	int16_t char_arr[MAX_NUMBER_OF_CHARS];
	int16_t size = 0, i = 0;
	int16_t step = 7;
	int16_t start = 0;
	int16_t sign = 1;

	if (value == -1)
		return;

	if (INVERTED_MODE)
		sign = -1;

	while (value > 0) {
		char_arr[size] = value % 10;
		size ++;
		value /= 10; 
	}

	switch (size) {
	case 1:
		start = 1;
		break;
	case 2:
		start = 3;
		break;
	case 3:
		start = 7;
		break;
	case 4:
		start = 10;
		break;
	case 5:
		start = 13;
		break;
	case 6:
		start = 16;
		break;
	case 7:
		start = 19;
		break;
	default:
		start = 7;
		break;
	}
	
	start = start * sign;

	for (i = size - 1; i >= 0; i--) {
		w_lcd_draw_char(pos.x - start, pos.y, numbers[char_arr[i]], Color565(0,0,0));
		start = start - sign * step;
	}
}
/*
void w_lcd_clear_effect(int16_t ls, int16_t le)
{
	int16_t i, j;

	fillScreen(ST7735_RED);

	_delay_ms(500);

	if (INVERTED_MODE) {
		// swap
		le = w_height - ls;
		ls = w_height - le;
	}

	for (j = 0; j < w_width; j++)
		for (i = ls; i < le; i++)
			drawPixel(j, i, Color565(255,255,255));
}
*/

void w_lcd_clear_lines(int16_t ls, int16_t le)
{
	int16_t i, j;
	int16_t copy_ls = ls;

	_delay_ms(100);

	if (INVERTED_MODE) {
		// swap
		ls = w_height - le;
		le = w_height - copy_ls;
	}

	for (j = 36; j < w_width - 35; j++)
		for (i = ls; i < le; i++)
			drawPixel(j, i, Color565(255, 255, 255));
}

// "press button" text effect
static int8_t ii = 0;
void w_lcd_print_press_btn_effect(enum t_bool first_time)
{
	struct t_point pos;
	// int8_t i = 0;
	int8_t step = 15;
	int8_t offset_x = 35;
	int8_t offset_y = 45;

	pos.x = offset_x;
	pos.y = offset_y + ii * step;

	if (!first_time) {
		w_lcd_clear_lines(pos.y - 5, pos.y + 4);
	}
	w_lcd_print_press_btn(pos);
	ii++;
	ii %= 4;
}

void w_lcd_print_press_btn(struct t_point pos)
{
	
	int16_t size = 0, i = 0;
	int16_t step = 7;
	int16_t start = 0;
	int16_t sign = 1;

	if (INVERTED_MODE) {
		sign = -1;
		pos.x = w_width - pos.x;
		pos.y = w_height - pos.y;
	}
	start = start * sign;

	for (i = 0; i < 9; i++) {
		w_lcd_draw_char(pos.x - start, pos.y, press_btn[i], Color565(0,0,0));
		start = start - sign * step;
		_delay_ms(50);
	}
}

void w_lcd_draw_char(int16_t x, int16_t y, int8_t char_matrix[7][7], uint16_t color)
{
	int16_t yy, xx;

	if (!INVERTED_MODE) {
		for (yy = 0; yy < 7; yy++) {
			for (xx = 0; xx < 7; xx++) {
				if(char_matrix[yy][xx] == 1)
					drawPixel(x - 3 + xx, y - 3 + yy, color);
			}
		}
	} else {
		for (yy = 0; yy < 7; yy++) {
			for (xx = 0; xx < 7; xx++) {
				if(char_matrix[yy][xx] == 1)
					drawPixel(x - 3 + 7 - xx, y - 3 + 7 - yy, color);
			}
		}
	}
}

int16_t get_inverted_x(int16_t x)
{
	return _width - x;
}

int16_t get_inverted_y(int16_t y)
{
	return _height - y;
}

struct t_point get_inverted_point(struct t_point p)
{
	struct t_point t;
	
	t.x = get_inverted_x(p.x);
	t.y = get_inverted_y(p.y);

	return t;
}

int16_t min(int16_t x, int16_t y)
{
	if (x < y)
		return x;
	return y;
}

int16_t max(int16_t x, int16_t y)
{
	if (x > y)
		return x;
	return y;
}
