#ifndef _W_LCD_H
#define _W_LCD_H

#include "lcd.h"
#include "util.h"

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdlib.h>

static int16_t w_height = 128;
static int16_t w_width = 128;

#define MAX_NUMBER_OF_CHARS		8

struct t_point {
	int16_t x, y;
};

void w_lcd_init();
void w_lcd_draw_square(struct t_point p1, struct t_point p2, uint16_t color, uint16_t border_color, int32_t value);

void w_lcd_drawL(int x, int y);
void w_lcd_print_value(struct t_point pos, int32_t value);
void w_lcd_draw_char(int16_t x, int16_t y, int8_t char_matrix[7][7], uint16_t color);
void w_lcd_clear_screen();
void w_lcd_print_press_btn_effect(enum t_bool first_time);
void w_lcd_print_press_btn();

int16_t get_inverted_x(int16_t x);
int16_t get_inverted_y(int16_t y);
struct t_point get_inverted_point(struct t_point p);
int16_t min(int16_t x, int16_t y);
int16_t max(int16_t x, int16_t y);

static int8_t numbers[10][7][7] = {
	  { 
	    { 0, 0, 1, 1, 1, 0, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 0, 1, 1, 1, 0, 0} 
	},{  
		{ 0, 0, 1, 1, 1, 0, 0},
		{ 0, 0, 0, 1, 0, 0, 0},
		{ 0, 0, 0, 1, 0, 0, 0},
		{ 0, 0, 0, 1, 0, 0, 0},
		{ 0, 0, 0, 1, 0, 0, 0},
		{ 0, 0, 0, 1, 0, 0, 0},
		{ 0, 0, 1, 1, 1, 0, 0}
	},{
	    { 0, 1, 1, 1, 1, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 1, 1, 1, 1, 0} 
	},{
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 1, 0} 
	},{
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0} 
	},{
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 1, 0}
	},{
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 1, 0} 
	},{
		{ 0, 1, 1, 1, 1, 0, 0},
		{ 0, 0, 0, 0, 1, 0, 0},
		{ 0, 0, 0, 0, 1, 0, 0},
		{ 0, 0, 1, 1, 1, 1, 0},
		{ 0, 0, 0, 0, 1, 0, 0},
		{ 0, 0, 0, 0, 1, 0, 0},
		{ 0, 0, 0, 0, 1, 0, 0} 
	},{ 
		{ 0, 0, 1, 1, 1, 0, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 0, 1, 1, 1, 0, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 0, 1, 1, 1, 0, 0} 
	},{
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 1, 0} 
	}
};

static int8_t press_btn[9][7][7] = {
	{ 
		{ 0, 1, 1, 1, 1, 0, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 0, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 0, 0, 0, 0, 0} 
	},{
		{ 0, 1, 1, 1, 1, 0, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 0, 0},
		{ 0, 1, 0, 1, 0, 0, 0},
		{ 0, 1, 0, 0, 1, 0, 0},
		{ 0, 1, 0, 0, 0, 1, 0} 
	}, {
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 1, 1, 1, 0, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 1, 1, 1, 1, 0} 
	}, {
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 1, 0}
	}, {
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 0, 0, 0, 0, 0},
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 0, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 1, 0}
	}, {
		{ 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0},
		{ 0, 0, 0, 0, 0, 0, 0}
	}, {
		{ 0, 1, 1, 1, 1, 0, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 0, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 1, 1, 0, 0}
	}, {
		{ 0, 1, 1, 1, 1, 1, 0},
		{ 0, 0, 0, 1, 0, 0, 0},
		{ 0, 0, 0, 1, 0, 0, 0},
		{ 0, 0, 0, 1, 0, 0, 0},
		{ 0, 0, 0, 1, 0, 0, 0},
		{ 0, 0, 0, 1, 0, 0, 0},
		{ 0, 0, 0, 1, 0, 0, 0}
	}, {
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 1, 0, 0, 1, 0},
		{ 0, 1, 0, 1, 0, 1, 0},
		{ 0, 1, 0, 0, 1, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0},
		{ 0, 1, 0, 0, 0, 1, 0}
	}
};

#endif